@@ -43,12 +43,8 @@
 #endif
 
 #if XORG < 118
-#if XORG < 110
-#define GetMaster(dev, type) ((dev)->u.master)
-#else
 #define GetMaster(dev, type) ((dev)->master)
 #endif
-#endif
 
 extern DeviceIntPtr vncKeyboardDev;
 
