@@ -148,10 +148,7 @@ void vncExtensionInit(void)
   vncExtGeneration = vncGetServerGeneration();
 
   if (vncGetScreenCount() > MAXSCREENS)
-    vncFatalError("vncExtensionInit: too many screens");
-
-  if (sizeof(ShortRect) != sizeof(struct UpdateRect))
-    vncFatalError("vncExtensionInit: Incompatible ShortRect size");
+    vncFatalError("vncExtensionInit: too many screens\n");
 
   vncAddExtension();
 
@@ -235,7 +232,7 @@ void vncExtensionInit(void)
       vncHooksInit(scr);
     }
   } catch (rdr::Exception& e) {
-    vncFatalError("vncExtInit: %s",e.str());
+    vncFatalError("vncExtInit: %s\n",e.str());
   }
 
   vncRegisterBlockHandlers();
@@ -249,7 +246,7 @@ void vncExtensionClose(void)
       desktop[scr] = NULL;
     }
   } catch (rdr::Exception& e) {
-    vncFatalError("vncExtInit: %s",e.str());
+    vncFatalError("vncExtInit: %s\n",e.str());
   }
 }
 
@@ -372,25 +369,24 @@ void vncSetLEDState(unsigned long leds)
     desktop[scr]->setLEDState(state);
 }
 
-void vncAddChanged(int scrIdx, const struct UpdateRect *extents,
-                   int nRects, const struct UpdateRect *rects)
+void vncAddChanged(int scrIdx, int nRects,
+                   const struct UpdateRect *rects)
 {
-  Region reg;
-
-  reg.setExtentsAndOrderedRects((const ShortRect*)extents,
-                                nRects, (const ShortRect*)rects);
-  desktop[scrIdx]->add_changed(reg);
+  for (int i = 0;i < nRects;i++) {
+    desktop[scrIdx]->add_changed(Region(Rect(rects[i].x1, rects[i].y1,
+                                             rects[i].x2, rects[i].y2)));
+  }
 }
 
-void vncAddCopied(int scrIdx, const struct UpdateRect *extents,
-                  int nRects, const struct UpdateRect *rects,
+void vncAddCopied(int scrIdx, int nRects,
+                  const struct UpdateRect *rects,
                   int dx, int dy)
 {
-  Region reg;
-
-  reg.setExtentsAndOrderedRects((const ShortRect*)extents,
-                                nRects, (const ShortRect*)rects);
-  desktop[scrIdx]->add_copied(reg, rfb::Point(dx, dy));
+  for (int i = 0;i < nRects;i++) {
+    desktop[scrIdx]->add_copied(Region(Rect(rects[i].x1, rects[i].y1,
+                                            rects[i].x2, rects[i].y2)),
+                                Point(dx, dy));
+  }
 }
 
 void vncSetCursor(int width, int height, int hotX, int hotY,
@@ -413,8 +409,13 @@ void vncPostScreenResize(int scrIdx, int success, int width, int height)
 {
   if (success) {
     // Let the RFB core know of the new dimensions and framebuffer
-    desktop[scrIdx]->setFramebuffer(width, height,
-                                    vncFbptr[scrIdx], vncFbstride[scrIdx]);
+    try {
+      desktop[scrIdx]->setFramebuffer(width, height,
+                                      vncFbptr[scrIdx],
+                                      vncFbstride[scrIdx]);
+    } catch (rdr::Exception& e) {
+      vncFatalError("vncPostScreenResize: %s\n", e.str());
+    }
   }
 
   desktop[scrIdx]->unblockUpdates();
@@ -430,7 +431,7 @@ void vncRefreshScreenLayout(int scrIdx)
   try {
     desktop[scrIdx]->refreshScreenLayout();
   } catch (rdr::Exception& e) {
-    vncFatalError("%s", e.str());
+    vncFatalError("vncRefreshScreenLayout: %s\n", e.str());
   }
 }
 
