@@ -22,6 +22,13 @@
 int cocoa_capture_display(Fl_Window *win, bool all_displays);
 void cocoa_release_display(Fl_Window *win);
 
+typedef struct CGColorSpace *CGColorSpaceRef;
+
+CGColorSpaceRef cocoa_win_color_space(Fl_Window *win);
+
+bool cocoa_win_is_zoomed(Fl_Window *win);
+void cocoa_win_zoom(Fl_Window *win);
+
 int cocoa_is_keyboard_event(const void *event);
 
 int cocoa_is_key_press(const void *event);
@@ -29,4 +36,10 @@ int cocoa_is_key_press(const void *event);
 int cocoa_event_keycode(const void *event);
 int cocoa_event_keysym(const void *event);
 
+int cocoa_set_caps_lock_state(bool on);
+int cocoa_set_num_lock_state(bool on);
+
+int cocoa_get_caps_lock_state(bool *on);
+int cocoa_get_num_lock_state(bool *on);
+
 #endif
